@@ -107,4 +107,54 @@ NULL
 #' @keywords data
 "data_corpus_inaugural"
 
+#' Lexicoder Sentiment Dictionary (2015)
+#' 
+#' The 2015 Lexicoder Sentiment Dictionary in \pkg{quanteda} \link{dictionary}
+#' format.  
+#' 
+#' @details
+#' The dictionary consists of 2,858 "negative" sentiment words and 1,709
+#' "positive" sentiment words. A further set of 2,860 and 1,721 negations of
+#' negative and positive words, respectively, is also included. While many users
+#' will find the non-negation sentiment forms of the LSD adequate for sentiment
+#' analysis, Young and Soroka (2012) did find a small, but non-negligible
+#' increase in performance when accounting for negations.  Users wishing to test
+#' this or include the negations are encouraged to subtract negated positive
+#' words from the count of positive words, and subtract the negated negative
+#' words from the negative count.  
+#' 
+#' Young and Soroka (2012) also suggest the use of a pre-processing script to
+#' remove specific cases of some words (i.e., "good bye", or "nobody better",
+#' which should not be counted as positive). Pre-processing scripts are
+#' available at \url{http://lexicoder.com}.
+#' @section License and Conditions: 
+#'   The LSD is available for non-commercial academic purposes only. By using
+#'   \code{data_dictionary_LSD2015}, you accept these terms.
+#'   
+#'   Please cite the references below when using the dictionary.
+#' @references 
+#'   The objectives, development and reliability of the dictionary are discussed
+#'   in detail in Young and Soroka (2012). Please cite this article when using the
+#'   Lexicoder Sentiment Dictionary and related resources.
+#
+#'   Young, Lori and Stuart Soroka. 2012. \emph{Lexicoder Sentiment
+#'   Dictionary}. Available at \url{http://lexicoder.com}.
+#'   
+#'   Young, Lori and Stuart Soroka.  2012.  "Affective News: The Automated
+#'   Coding of Sentiment in Political Texts." \emph{Political Communication} 29(2):
+#'   205-231.
+#' @keywords data
+#' @examples 
+#' # simple example
+#' txt <- "This aggressive policy will not win friends."
+#' tokens_lookup(tokens(txt), dictionary = data_dictionary_LSD2015, exclusive = FALSE)
+#' ## tokens from 1 document.
+#' ## text1 :
+#' ## [1] "This"   "NEGATIVE"   "policy"   "will"   "NEG_POSITIVE" "POSITIVE" "."
+#' 
+#' # on larger examples - notice that few negations are used
+#' dfm(data_char_ukimmig2010, dictionary = data_dictionary_LSD2015)
+#' kwic(data_char_ukimmig2010, "not")
+"data_dictionary_LSD2015"
+
 
