@@ -9,8 +9,8 @@
 #'   documents, you need to slice out the document(s) from the dfm or tokens
 #'   object.
 #'   
-#'   Comparison word cloud plots may be plotted by setting \code{comparison = 
-#'   TRUE}, which plots a separate grouping for \emph{each document} in the dfm.
+#'   Comparison wordcloud plots may be plotted by setting \code{comparison = TRUE}, 
+#'   which plots a separate grouping for \emph{each document} in the dfm.
 #'   This means that you will need to slice out just a few documents from the 
 #'   dfm, or to create a dfm where the "documents" represent a subset or a 
 #'   grouping of documents by some document variable.
@@ -44,13 +44,12 @@
 #' @export
 #' @keywords plot
 textplot_wordcloud <- function(x, comparison = FALSE, ...) {
-    
-    if (!is.dfm(x) & !is.tokens(x))
-        stop("x must be a dfm or tokens object")
-    
-    if (is.tokens(x))
-        x <- dfm(x, verbose = FALSE)
-    
+    UseMethod("textplot_wordcloud")
+}
+
+#' @noRd
+#' @export
+textplot_wordcloud.dfm <- function(x, comparison = FALSE, ...) {
     if (comparison) {
         if (ndoc(x) > 8) stop("Too many documents to plot comparison, use 8 or fewer documents.")
         wordcloud::comparison.cloud(t(as.matrix(x)), ...)
@@ -59,4 +58,10 @@ textplot_wordcloud <- function(x, comparison = FALSE, ...) {
     }
 }
 
+#' @noRd
+#' @export
+textplot_wordcloud.tokenizedTexts <- function(x, comparison = FALSE, ...) {
+    textplot_wordcloud(dfm(x, verbose = FALSE), comparison = comparison, ...)
+}
+    
 
