@@ -1,3 +1,7 @@
+#' @importFrom magrittr %>%
+#' @export
+magrittr::`%>%`
+
 # rdname catm
 # make temporary files and directories in a more reasonable way than tempfile()
 # or tempdir(): here, the filename is different each time you call mktemp()
@@ -80,12 +84,11 @@ reassign_slots <- function(x_new, x_orig, exceptions = NULL) {
 }
 
 
-#' R-like alternative to reassign_attributes()
-#' @keywords internal
+#' function extending base::attributes()
 #' @param x an object
 #' @param overwrite if \code{TRUE}, overwrite old attributes
 #' @param value new attributes
-#' @author Kohei Watanabe
+#' @keywords internal
 "attributes<-" <- function(x, overwrite = TRUE, value) {
     if (overwrite) {
         base::attributes(x) <- value
@@ -95,6 +98,21 @@ reassign_slots <- function(x_new, x_orig, exceptions = NULL) {
     return(x)
 }
 
+#' function to assign multiple slots to a S4 object
+#' @param x an S4 object
+#' @param exceptions slots to ignore
+#' @param value a list of attributes extracted by attributes()
+#' @keywords internal
+"slots<-" <- function(x, exceptions = c("Dim", "Dimnames", "i", "p", "x", "factors"), value) {
+    slots <- methods::getSlots(class(x)[1])
+    for (sname in names(value)) {
+        if (!sname %in% names(slots) || sname %in% exceptions) next
+        if (!identical(typeof(value[[sname]]), slots[[sname]])) next
+        methods::slot(x, sname) <- value[[sname]]
+    }
+    return(x)
+}
+
 #' utility function to remove all attributes
 #' @keywords internal
 remove_attributes <- function(x) {
@@ -103,15 +121,17 @@ remove_attributes <- function(x) {
 }
 
 #' utility function to create a object with new set of attributes
+#' @param x an underlying R object of a new object
+#' @param attrs attributes of a new object
+#' @param overwrite_attributes overwrite attributes of the input object, if \code{TRUE}
 #' @keywords internal
-create <- function(x, what, attrs = NULL, ...) {
-    base::attributes(x) <- NULL
+create <- function(x, what, attrs = NULL, overwrite_attributes = FALSE, ...) {
     if (what == 'tokens') {
         class <- c('tokens', 'tokenizedTexts', 'list')
     }
     x <- structure(x, class = class, ...)
     if (!is.null(attrs)) {
-        attributes(x, FALSE) <- attrs
+        attributes(x, overwrite_attributes) <- attrs
     }
     return(x)
 }
@@ -141,31 +161,31 @@ texts_random <- function(n_doc=10,
                          code=FALSE,
                          seed, characters){
     
-    if(!missing(seed)) set.seed(seed)
-    if(missing(characters)){
+    if (!missing(seed)) set.seed(seed)
+    if (missing(characters)){
         # Empirical distribution in English (https://en.wikipedia.org/wiki/Letter_frequency)
         chars <- letters
         prob_chars <-c(0.08167, 0.01492, 0.02782, 0.04253, 0.12702, 0.02228, 0.02015, 
                        0.06094, 0.06966, 0.00153, 0.00772, 0.04025, 0.02406, 0.06749,
                        0.07507, 0.01929, 0.00095, 0.05987, 0.06327, 0.09056, 0.02758, 
                        0.00978, 0.02360, 0.00150, 0.01974, 0.00074)
-    }else{
+    } else {
         # Log-normal distribution
         chars <- characters
         dist_chars <- stats::rlnorm(length(chars))
         prob_chars <- sort(dist_chars / sum(dist_chars), decreasing = TRUE)
     }
-    if(n_type > length(chars) ^ len_word) 
+    if (n_type > length(chars) ^ len_word) 
         stop('n_type is too large')
     
     # Generate unique types
     type <- c()
-    if(fast){
+    if (fast) {
         pat <- stri_flatten(c('[', chars, ']'))
         while(n_type > length(type)){
             type <- unique(c(type, stri_rand_strings(n_type, 1:len_word, pat)))
         }
-    }else{
+    } else {
         while(n_type > length(type)){
             type <- unique(c(type, word_random(chars, sample(len_word, 1), prob_chars)))
         }
@@ -179,9 +199,9 @@ texts_random <- function(n_doc=10,
         words <- sample(type, size=n_word, replace = TRUE, prob=prob_words)
         stri_c(words, collapse = ' ')
     })
-    if(code){
+    if (code) {
         return(code(texts))
-    }else{
+    } else {
         return(texts)
     }
 }
@@ -197,35 +217,61 @@ zipf <- function(n_type){
 code <- function(texts){
     len <- length(texts)
     cat(paste0('txt <- c("', texts[1], '",\n'))
-    for(text in texts[2:(len-1)]){
+    for (text in texts[2:(len-1)]) {
         cat(paste0('         "', text, '",\n'))
     }
     cat(paste0('         "', texts[len], '")\n'))
 }
 
 
-# convert various input as features to a vector used in tokens_select, 
-# tokens_compound and kwic.
-features2id <- function(features, types, valuetype, case_insensitive, 
-                        concatenator = '_', remove_unigram = FALSE) {
+#' convert various input as pattern to a vector used in tokens_select, 
+#' tokens_compound and kwic.
+#' @inheritParams pattern
+#' @inheritParams valuetype
+#' @param case_insensitive ignore the case of dictionary values if \code{TRUE}
+#' @param concatenator concatenator that join multi-word expression in tokens object
+#' @param remove_unigram ignore single-word patterns if \code{TRUE}
+#' @seealso regex2id
+#' @keywords internal
+pattern2id <- function(pattern, types, valuetype, case_insensitive, 
+                       concatenator = '_', remove_unigram = FALSE) {
     
-    if (is.sequences(features) || is.collocations(features)) {
-        features <- stri_split_charclass(features$collocation, "\\p{Z}")
-        features_id <- lapply(features, function(x) fastmatch::fmatch(x, types))
-        features_id <- features_id[sapply(features_id, function(x) all(!is.na(x)))]
+    if (is.sequences(pattern) || is.collocations(pattern)) {
+        pattern <- stri_split_charclass(pattern$collocation, "\\p{Z}")
+        pattern_id <- lapply(pattern, function(x) fastmatch::fmatch(x, types))
+        pattern_id <- pattern_id[sapply(pattern_id, function(x) all(!is.na(x)))]
     } else {
-        if (is.dictionary(features)) {
-            features <- unlist(features, use.names = FALSE)
-            features <- split_dictionary_values(features, concatenator)
+        if (is.dictionary(pattern)) {
+            pattern <- unlist(pattern, use.names = FALSE)
+            pattern <- split_dictionary_values(pattern, concatenator)
         } else {
-            features <- as.list(features)
+            pattern <- as.list(pattern)
         }
         if (remove_unigram)
-            features <- features[lengths(features) > 1] # drop single-word features
-        features_id <- regex2id(features, types, valuetype, case_insensitive)
+            pattern <- pattern[lengths(pattern) > 1] # drop single-word pattern
+        pattern_id <- regex2id(pattern, types, valuetype, case_insensitive)
     }
-    attr(features_id, 'features') <- stri_c_list(features, sep = ' ')
-    return(features_id)
+    attr(pattern_id, 'pattern') <- stri_c_list(pattern, sep = ' ')
+    return(pattern_id)
+}
+
+
+#' internal function for \code{select_types()} to check if a string is a regular expression
+#' @param x a character string to be tested
+#' @keywords internal
+is_regex <- function(x){
+    any(stri_detect_fixed(x, c(".", "(", ")", "^", "{", "}", "+", "$", "*", "?", "[", "]", "\\")))
+}
+
+#' internal function for \code{select_types()} to escape regular expressions 
+#' 
+#' This function escapes glob patterns before \code{utils:glob2rx()}, therefore * and ?
+#' are unescaped.
+#' @param x character vector to be escaped
+#' @keywords internal
+escape_regex <- function(x){
+    #stri_replace_all_regex(x, "([.()^\\{\\}+$*\\[\\]\\\\])", "\\\\$1") # escape any
+    stri_replace_all_regex(x, "([.()^\\{\\}+$\\[\\]\\\\])", "\\\\$1") # allow glob
 }
 
 # which object class started the call stack?
@@ -242,3 +288,14 @@ who_called_me_first <- function(x, function_name) {
     x[1]
 }
 
+# function to check dots arguments against a list of permissible arguments
+check_dots <-  function(dots, permissible_args = NULL) {
+    if (length(dots) == 0) return()
+    args <- names(dots)
+    impermissible_args <-  setdiff(args, permissible_args)
+    if (length(impermissible_args))
+        warning("Argument", if (length(impermissible_args) > 1) "s " else " ", 
+                paste(impermissible_args, collapse = ', '), " not used.", 
+                noBreaks. = TRUE, call. = FALSE)
+}
+
