@@ -184,17 +184,17 @@ test_that("test textstat_dist method = \"Canberra\" against proxy dist() : featu
 
 # Hamming distance
 test_that("test textstat_dist method = \"hamming\" against e1071::hamming.distance: documents", {
-    presDfm <- dfm(corpus_subset(data_corpus_inaugural, Year > 1980), remove = stopwords("english"),
+    presDfm <- dfm(corpus_subset(data_corpus_inaugural, Year > 1980 & Year < 2018), remove = stopwords("english"),
                    stem = TRUE, verbose = FALSE)
     
     hammingQuanteda <- sort(as.matrix(textstat_dist(presDfm, "1981-Reagan", method = "hamming", margin = "documents", upper = TRUE))[,"1981-Reagan"], decreasing = FALSE)
     hammingQuanteda <- hammingQuanteda[-which(names(hammingQuanteda) == "1981-Reagan")]
     
-    if (requireNamespace("e1071", quietly = TRUE)){
+    if (requireNamespace("e1071", quietly = TRUE)) {
         hammingE1071 <- sort(e1071::hamming.distance(as.matrix(tf(presDfm, "boolean")))[, "1981-Reagan"], decreasing = FALSE)
         if("1981-Reagan" %in% names(hammingE1071)) hammingE1071 <- hammingE1071[-which(names(hammingE1071) == "1981-Reagan")]
     } else {
-        hammingE1071 <- c(711, 724, 745, 766, 766, 778, 785, 804, 851)
+        hammingE1071 <- c(712, 723, 746, 769, 774, 781, 784, 812, 857)
     }
     expect_equivalent(hammingQuanteda, hammingE1071)
 })
@@ -235,9 +235,10 @@ test_that("as.list.dist works as expected",{
     presDfm <- dfm(corpus_subset(data_corpus_inaugural, Year > 1980), remove = stopwords("english"),
                    stem = TRUE, verbose = FALSE)
     ddist <- textstat_dist(presDfm, method = "hamming")
-    ddist_list <- as.list(ddist)
-    expect_equal(names(ddist_list$`1981-Reagan`)[1:3], c("2009-Obama", "2013-Obama", "1997-Clinton"))
-    expect_equivalent(ddist_list$`1981-Reagan`[1:3], c(851, 804, 785))
+    expect_equal(
+        as.list(ddist)$`1981-Reagan`[1:3], 
+        c("2009-Obama" = 857, "2013-Obama" = 812, "1997-Clinton" = 784)
+    )
 })
 
 test_that("as.list.dist.selection works as expected",{
